@@ -18,7 +18,7 @@
 
 modname = distname = 'pylint'
 
-numversion = (0, 21, 0)
+numversion = (0, 21, 1)
 version = '.'.join([str(num) for num in numversion])
 
 
@@ -46,7 +46,7 @@ classifiers =  ['Development Status :: 4 - Beta',
                 ]
 
 
-short_desc = "python code static checker"
+description = "python code static checker"
 long_desc = """\
  Pylint is a Python source code analyzer which looks for programming
  errors, helps enforcing a coding standard and sniffs for some code
