@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2009 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2010 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -18,12 +18,23 @@
 from __future__ import generators
 
 from logilab import astng
-from logilab.astng.infutils import YES, Instance, are_exclusive
+from logilab.astng import YES, Instance, are_exclusive
 
 from pylint.interfaces import IASTNGChecker
 from pylint.checkers import BaseChecker
 from pylint.checkers.utils import PYMETHODS, overrides_a_method
 
+def class_is_abstract(node):
+    """return true if the given class node should be considered as an abstract
+    class
+    """
+    for method in node.methods():
+        if method.parent.frame() is node:
+            if method.is_abstract(pass_is_abstract=False):
+                return True
+    return False
+
+
 MSGS = {
     'F0202': ('Unable to check methods signature (%s / %s)',
               'Used when PyLint has been unable to check methods signature \
@@ -179,13 +190,20 @@ instance attributes.'}
                     isinstance(n.statement(), (astng.Delete, astng.AugAssign))]
             if not nodes:
                 continue # error detected by typechecking
-            node = nodes[0] # XXX
-            frame = node.frame()
-            if frame.name not in defining_methods:
+            attr_defined = False
+            # check if any method attr is defined in is a defining method
+            for node in nodes:
+                if node.frame().name in defining_methods:
+                    attr_defined = True
+            if not attr_defined:
                 # check attribute is defined in a parent's __init__
                 for parent in cnode.instance_attr_ancestors(attr):
-                    frame = parent.instance_attrs[attr][0].frame() # XXX
-                    if frame.name in defining_methods:
+                    attr_defined = False
+                    # check if any parent method attr is defined in is a defining method
+                    for node in parent.instance_attrs[attr]:
+                        if node.frame().name in defining_methods:
+                            attr_defined = True
+                    if attr_defined:
                         # we're done :)
                         break
                 else:
@@ -298,7 +316,7 @@ instance attributes.'}
         # XXX refactor, probably much simpler now that E0201 is in type checker
         for attr, nodes in accessed.items():
             # deactivate "except doesn't do anything", that's expected
-            # pylint: disable-msg=W0704
+            # pylint: disable=W0704
             # is it a class attribute ?
             try:
                 node.local_attr(attr)
@@ -369,6 +387,9 @@ instance attributes.'}
         """check that the given class node implements abstract methods from
         base classes
         """
+        # check if this class abstract
+        if class_is_abstract(node):
+            return
         for method in node.methods():
             owner = method.parent.frame()
             if owner is node:
@@ -433,6 +454,7 @@ instance attributes.'}
         """
         klass_node = node.parent.frame()
         to_call = _ancestors_to_call(klass_node)
+        not_called_yet = dict(to_call)
         for stmt in node.nodes_of_class(astng.CallFunc):
             expr = stmt.func
             if not isinstance(expr, astng.Getattr) \
@@ -448,12 +470,13 @@ instance attributes.'}
                 if klass is YES:
                     continue
                 try:
-                    del to_call[klass]
+                    del not_called_yet[klass]
                 except KeyError:
-                    self.add_message('W0233', node=expr, args=klass.name)
+                    if klass not in to_call:
+                        self.add_message('W0233', node=expr, args=klass.name)
             except astng.InferenceError:
                 continue
-        for klass in to_call.keys():
+        for klass in not_called_yet.keys():
             if klass.name == 'object':
                 continue
             self.add_message('W0231', args=klass.name, node=node)
