@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2009 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2010 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -21,7 +21,7 @@ from logilab.common.ureports import VerbatimText, Paragraph
 from logilab.common.compat import sorted, enumerate
 
 from logilab import astng
-from logilab.astng.infutils import are_exclusive
+from logilab.astng import are_exclusive
 
 from pylint.interfaces import IASTNGChecker
 from pylint.checkers import BaseChecker, EmptyReport
@@ -59,7 +59,7 @@ def filter_dependencies_info(dep_info, package_dir, mode='external'):
 
 def make_tree_defs(mod_files_list):
     """get a list of 2-uple (module, list_of_files_which_import_this_module),
-    it will return a dictionnary to represent this as a tree
+    it will return a dictionary to represent this as a tree
     """
     tree_defs = {}
     for mod, files in mod_files_list:
@@ -173,21 +173,21 @@ separated by a comma'}
                  'type' : 'string',
                  'metavar' : '<file.dot>',
                  'help' : 'Create a graph of every (i.e. internal and \
-external) dependencies in the given file (report R0402 must not be disabled)'}
+external) dependencies in the given file (report RP0402 must not be disabled)'}
                 ),
                ('ext-import-graph',
                 {'default' : '',
                  'type' : 'string',
                  'metavar' : '<file.dot>',
                  'help' : 'Create a graph of external dependencies in the \
-given file (report R0402 must not be disabled)'}
+given file (report RP0402 must not be disabled)'}
                 ),
                ('int-import-graph',
                 {'default' : '',
                  'type' : 'string',
                  'metavar' : '<file.dot>',
                  'help' : 'Create a graph of internal dependencies in the \
-given file (report R0402 must not be disabled)'}
+given file (report RP0402 must not be disabled)'}
                 ),
 
                )
@@ -197,9 +197,9 @@ given file (report R0402 must not be disabled)'}
         self.stats = None
         self.import_graph = None
         self.__int_dep_info = self.__ext_dep_info = None
-        self.reports = (('R0401', 'External dependencies',
+        self.reports = (('RP0401', 'External dependencies',
                          self.report_external_dependencies),
-                        ('R0402', 'Modules dependencies graph',
+                        ('RP0402', 'Modules dependencies graph',
                          self.report_dependencies_graph),
                         )
 
@@ -213,9 +213,9 @@ given file (report R0402 must not be disabled)'}
     def close(self):
         """called before visiting project (i.e set of modules)"""
         # don't try to compute cycles if the associated message is disabled
-        if self.linter.is_message_enabled('R0401'):
+        if self.linter.is_message_enabled('RP0401'):
             for cycle in get_cycles(self.import_graph):
-                self.add_message('R0401', args=' -> '.join(cycle))
+                self.add_message('RP0401', args=' -> '.join(cycle))
 
     def visit_import(self, node):
         """triggered when an import statement is seen"""
