@@ -1,4 +1,4 @@
-# Copyright (c) 2006-2009 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2006-2010 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -20,8 +20,7 @@ from logilab.common.compat import set
 
 
 from logilab import astng
-from logilab.astng import InferenceError, NotFoundError
-from logilab.astng.infutils import YES, Instance
+from logilab.astng import InferenceError, NotFoundError, YES, Instance
 
 from pylint.interfaces import IASTNGChecker
 from pylint.checkers import BaseChecker
@@ -124,7 +123,7 @@ accessed.'}
 
         function/method, super call and metaclasses are ignored
         """
-        if node.attrname in self.config.generated_members:
+        if node.attrname in self.generated_members:
             # attribute is marked as generated, stop here
             return
         try:
