@@ -1,6 +1,6 @@
-# pylint: disable-msg=W0611
+# pylint: disable=W0611
 #
-# Copyright (c) 2003-2009 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2010 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -15,7 +15,7 @@
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""some functions that may be usefull for various checkers
+"""some functions that may be useful for various checkers
 """
 
 from logilab import astng
@@ -29,9 +29,9 @@ except AttributeError:
     FOR_NODE_TYPES = (astng.For, astng.Comprehension)
 
 def safe_infer(node):
-    """return the infered value for the given node.
+    """return the inferred value for the given node.
     Return None if inference failed or if there is some ambiguity (more than
-    one node has been infered)
+    one node has been inferred)
     """
     try:
         inferit = node.infer()
@@ -40,7 +40,7 @@ def safe_infer(node):
         return
     try:
         inferit.next()
-        return # None if there is ambiguity on the infered node
+        return # None if there is ambiguity on the inferred node
     except StopIteration:
         return value
 
@@ -99,6 +99,9 @@ def is_defined_before(var_node, comp_node_types=COMP_NODE_TYPES):
                 if ass_node.name == varname:
                     return True
         elif isinstance(_node, astng.With):
+            if _node.vars is None:
+                # quickfix : case in which 'with' is used without 'as'
+                return False
             if _node.vars.name == varname:
                 return True
         elif isinstance(_node, (astng.Lambda, astng.Function)):
@@ -182,18 +185,10 @@ def overrides_a_method(class_node, name):
             return True
     return False
 
-def display_type(node):
-    """return the type of this node for screen display"""
-    if isinstance(node, astng.Instance):
-        return 'Instance of'
-    elif isinstance(node, astng.Module):
-        return 'Module'
-    return 'Class'
-
 PYMETHODS = set(('__new__', '__init__', '__del__', '__hash__',
                  '__str__', '__repr__',
                  '__len__', '__iter__',
-                 '__get__', '__set__',
+                 '__delete__', '__get__', '__set__',
                  '__getitem__', '__setitem__', '__delitem__', '__contains__',
                  '__getattribute__', '__getattr__', '__setattr__', '__delattr__',
                  '__call__',
