@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2009 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2010 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -20,8 +20,7 @@ from copy import copy
 
 from logilab.common.compat import enumerate
 from logilab import astng
-from logilab.astng.lookup import builtin_lookup
-from logilab.astng.infutils import are_exclusive
+from logilab.astng import are_exclusive, builtin_lookup
 
 from pylint.interfaces import IASTNGChecker
 from pylint.checkers import BaseChecker
@@ -30,7 +29,7 @@ from pylint.checkers.utils import PYMETHODS, is_ancestor_name, is_builtin, \
 
 
 def overridden_method(klass, name):
-    """get overriden method if any"""
+    """get overridden method if any"""
     try:
         parent = klass.local_attr_ancestors(name).next()
     except (StopIteration, KeyError):
@@ -59,9 +58,9 @@ MSGS = {
     'W0601': ('Global variable %r undefined at the module level',
               'Used when a variable is defined through the "global" statement \
               but the variable is not defined in the module scope.'),
-    'W0602': ('Using global for %r but no assigment is done',
+    'W0602': ('Using global for %r but no assignment is done',
               'Used when a variable is defined through the "global" statement \
-              but no assigment to this variable is done.'),
+              but no assignment to this variable is done.'),
     'W0603': ('Using the global statement', # W0121
               'Used when you use the "global" statement to update a global \
               variable. PyLint just try to discourage this \
@@ -96,7 +95,7 @@ class VariablesChecker(BaseChecker):
     * unused variables / imports
     * undefined variables
     * redefinition of variable from builtins or from an outer scope
-    * use of variable before assigment
+    * use of variable before assignment
     """
 
     __implements__ = IASTNGChecker
@@ -107,7 +106,7 @@ class VariablesChecker(BaseChecker):
     options = (
                ("init-import",
                 {'default': 0, 'type' : 'yn', 'metavar' : '<y_or_n>',
-                 'help' : 'Tells wether we should check for unused import in \
+                 'help' : 'Tells whether we should check for unused import in \
 __init__ files.'}),
                ("dummy-variables-rgx",
                 {'default': ('_|dummy'),
@@ -240,7 +239,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                         overridden = overridden_method(klass, node.name)
                     if overridden is not None and name in overridden.argnames():
                         continue
-                    if node.name in PYMETHODS:
+                    if node.name in PYMETHODS and node.name not in ('__init__', '__new__'):
                         continue
                 # don't check callback arguments XXX should be configurable
                 if node.name.startswith('cb_') or node.name.endswith('_cb'):
@@ -264,18 +263,25 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                 # unassigned global, skip
                 assign_nodes = []
             for anode in assign_nodes:
+                if anode.parent is None:
+                    # node returned for builtin attribute such as __file__,
+                    # __doc__, etc...
+                    continue
                 if anode.frame() is frame:
-                    # same scope level assigment
+                    # same scope level assignment
                     break
             else:
-                # global but no assigment
+                # global but no assignment
                 self.add_message('W0602', args=name, node=node)
                 default_message = False
             if not assign_nodes:
                 continue
             for anode in assign_nodes:
+                if anode.parent is None:
+                    self.add_message('W0622', args=name, node=node)
+                    break
                 if anode.frame() is module:
-                    # module level assigment
+                    # module level assignment
                     break
             else:
                 # global undefined at the module scope
@@ -321,10 +327,12 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         """check that a name is defined if the current scope and doesn't
         redefine a built-in
         """
-        name = node.name
         stmt = node.statement()
-        # probably "is_statement == True" missing somewhere in astng
-        assert stmt.fromlineno, (stmt, node, node.fromlineno)
+        if stmt.fromlineno is None:
+            # name node from a astng built from live code, skip
+            assert not stmt.root().file.endswith('.py')
+            return
+        name = node.name
         frame = stmt.scope()
         # if the name node is used as a function default argument's value or as
         # a decorator, then start from the parent frame of the function instead
@@ -335,13 +343,17 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         else:
             start_index = len(self._to_consume) - 1
         # iterates through parent scopes, from the inner to the outer
+        base_scope_type = self._to_consume[start_index][-1]
         for i in range(start_index, -1, -1):
             to_consume, consumed, scope_type = self._to_consume[i]
             # if the current scope is a class scope but it's not the inner
             # scope, ignore it. This prevents to access this scope instead of
             # the globals one in function members when there are some common
-            # names
-            if scope_type == 'class' and i != start_index:
+            # names. The only exception is when the starting scope is a
+            # genexpr and its direct outer scope is a class
+            if scope_type == 'class' and i != start_index and not (
+                base_scope_type == 'genexpr' and i == start_index-1):
+                # XXX find a way to handle class scope in a smoother way
                 continue
             # the name has already been consumed, only check it's not a loop
             # variable used outside the loop
@@ -355,7 +367,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             except KeyError:
                 continue
             else:
-                # checks for use before assigment
+                # checks for use before assignment
                 defnode = assign_parent(to_consume[name][0])
                 if defnode is not None:
                     defstmt = defnode.statement()
@@ -380,7 +392,8 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                         if defstmt is stmt and isinstance(node, (astng.DelName,
                                                                  astng.AssName)):
                             self.add_message('E0602', args=name, node=node)
-                        else:
+                        elif self._to_consume[-1][-1] != 'lambda':
+                            # E0601 may *not* occurs in lambda scope
                             self.add_message('E0601', args=name, node=node)
                 if not isinstance(node, astng.AssName): # Aug AssName
                     del to_consume[name]
