@@ -65,8 +65,8 @@ def save_results(results, base):
     data_file = get_pdata_path(base, 1)
     try:
         pickle.dump(results, open(data_file, 'w'))
-    except OSError:
-        print >> sys.stderr, 'Unable to create file %s' % data_file
+    except (IOError, OSError), ex:
+        print >> sys.stderr, 'Unable to create file %s: %s' % (data_file, ex)
     
 # location of the configuration file ##########################################
 
@@ -86,10 +86,11 @@ def find_pylintrc():
     if os.environ.has_key('PYLINTRC') and exists(os.environ['PYLINTRC']):
         pylintrc = os.environ['PYLINTRC']
     else:
-        if USER_HOME == '~' or USER_HOME == '/root':
+        user_home = expanduser('~')
+        if user_home == '~' or user_home == '/root':
             pylintrc = ".pylintrc"
         else:
-            pylintrc = join(USER_HOME, '.pylintrc')
+            pylintrc = join(user_home, '.pylintrc')
     if not isfile(pylintrc):
         if isfile('/etc/pylintrc'):
             pylintrc = '/etc/pylintrc'
@@ -107,7 +108,7 @@ found, it defaults to ~/.pylint.d/ or .pylint.d (in the current working
 directory) . The current PYLINTHOME is %(PYLINT_HOME)s.        
     * PYLINTRC                                                                 
     path to the configuration file. If not found, it will use the first        
-existant file in ~/.pylintrc, /etc/pylintrc. The current PYLINTRC is
+existent file in ~/.pylintrc, /etc/pylintrc. The current PYLINTRC is
 %(PYLINTRC)s.                                                                 
 ''' % globals()
 
