@@ -34,6 +34,7 @@ import sys
 import os
 import re
 import tokenize
+from warnings import warn
 
 from logilab.common.configuration import UnsupportedAction, OptionsManagerMixIn
 from logilab.common.optik_ext import check_csv
@@ -41,14 +42,15 @@ from logilab.common.modutils import load_module_from_name
 from logilab.common.interface import implements
 from logilab.common.textutils import splitstrip
 from logilab.common.fileutils import norm_open
-from logilab.common.ureports import Table, Text
+from logilab.common.ureports import Table, Text, Section
+from logilab.common.graph import ordered_nodes
 from logilab.common.__pkginfo__ import version as common_version
 
 from logilab.astng import MANAGER, nodes
 from logilab.astng.__pkginfo__ import version as astng_version
 
-from pylint.utils import UnknownMessage, MessagesHandlerMixIn, \
-     ReportsHandlerMixIn, MSG_TYPES, sort_checkers, expand_modules
+from pylint.utils import PyLintASTWalker, UnknownMessage, MessagesHandlerMixIn,\
+     ReportsHandlerMixIn, MSG_TYPES, expand_modules
 from pylint.interfaces import ILinter, IRawChecker, IASTNGChecker
 from pylint.checkers import BaseRawChecker, EmptyReport, \
      table_lines_from_stats
@@ -67,6 +69,7 @@ REPORTER_OPT_MAP = {'text': TextReporter,
                     'colorized': ColorizedTextReporter,
                     'html': HTMLReporter,}
 
+
 # Python Linter class #########################################################
 
 MSGS = {
@@ -111,6 +114,7 @@ MSGS = {
               'Used when a bad value for an inline option is encountered.'),
     }
 
+
 class PyLinter(OptionsManagerMixIn, MessagesHandlerMixIn, ReportsHandlerMixIn,
                BaseRawChecker):
     """lint Python modules using external checkers.
@@ -122,10 +126,11 @@ class PyLinter(OptionsManagerMixIn, MessagesHandlerMixIn, ReportsHandlerMixIn,
     * handle some basic but necessary stats'data (number of classes, methods...)
     """
 
-    __implements__ = (ILinter, IRawChecker, IASTNGChecker)
+    __implements__ = (ILinter, IRawChecker)
 
     name = 'master'
     priority = 0
+    level = 0
     msgs = MSGS
     may_be_disabled = False
 
@@ -135,25 +140,14 @@ class PyLinter(OptionsManagerMixIn, MessagesHandlerMixIn, ReportsHandlerMixIn,
                  'help' : 'Add <file or directory> to the black list. It \
 should be a base name, not a path. You may set this option multiple times.'}),
 
-               ('enable-checker',
-                {'type' : 'csv', 'metavar': '<checker ids>',
-                 'group': 'Messages control',
-                 'help' : 'Enable only checker(s) with the given id(s).\
-                 This option conflicts with the disable-checker option'}),
-
-               ('disable-checker',
-                {'type' : 'csv', 'metavar': '<checker ids>',
-                 'group': 'Messages control',
-                 'help' : 'Enable all checker(s) except those with the \
-                 given id(s).\
-                 This option conflicts with the enable-checker option'}),
-
                ('persistent',
                 {'default': True, 'type' : 'yn', 'metavar' : '<y_or_n>',
+                 'level': 1,
                  'help' : 'Pickle collected data for later comparisons.'}),
 
                ('load-plugins',
                 {'type' : 'csv', 'metavar' : '<modules>', 'default' : (),
+                 'level': 1,
                  'help' : 'List of plugins (as comma separated values of \
 python modules names) to load, usually to register additional checkers.'}),
 
@@ -173,7 +167,7 @@ python modules names) to load, usually to register additional checkers.'}),
 
                ('files-output',
                 {'default': 0, 'type' : 'yn', 'metavar' : '<y_or_n>',
-                 'group': 'Reports',
+                 'group': 'Reports', 'level': 1,
                  'help' : 'Put messages in a separate file for each module / \
 package specified on the command line instead of printing them on stdout. \
 Reports (if any) will be written in a file name "pylint_global.[txt|html]".'}),
@@ -187,7 +181,7 @@ Reports (if any) will be written in a file name "pylint_global.[txt|html]".'}),
 
                ('evaluation',
                 {'type' : 'string', 'metavar' : '<python_expression>',
-                 'group': 'Reports',
+                 'group': 'Reports', 'level': 1,
                  'default': '10.0 - ((float(5 * error + warning + refactor + \
 convention) / statement) * 10)',
                  'help' : 'Python expression which should return a note less \
@@ -198,39 +192,25 @@ warning, statement which respectively contain the number of errors / warnings\
 
                ('comment',
                 {'default': 0, 'type' : 'yn', 'metavar' : '<y_or_n>',
-                 'group': 'Reports',
+                 'group': 'Reports', 'level': 1,
                  'help' : 'Add a comment according to your evaluation note. \
 This is used by the global evaluation report (R0004).'}),
 
-               ('enable-report',
-                {'type' : 'csv', 'metavar': '<rpt ids>',
-                 'group': 'Reports',
-                 'help' : 'Enable the report(s) with the given id(s).'}),
-
-               ('disable-report',
-                {'type' : 'csv', 'metavar': '<rpt ids>',
-                 'group': 'Reports',
-                 'help' : 'Disable the report(s) with the given id(s).'}),
-
-               ('enable-msg-cat',
-                {'type' : 'string', 'metavar': '<msg cats>',
-                 'group': 'Messages control',
-                 'help' : 'Enable all messages in the listed categories (IRCWEF).'}),
-
-               ('disable-msg-cat',
-                {'type' : 'string', 'metavar': '<msg cats>', 'default': 'I',
-                 'group': 'Messages control',
-                 'help' : 'Disable all messages in the listed categories (IRCWEF).'}),
-
-               ('enable-msg',
+               ('enable',
                 {'type' : 'csv', 'metavar': '<msg ids>',
+                 'short': 'e',
                  'group': 'Messages control',
-                 'help' : 'Enable the message(s) with the given id(s).'}),
+                 'help' : 'Enable the message, report, category or checker with the '
+                 'given id(s). You can either give multiple identifier '
+                 'separated by comma (,) or put this option multiple time.'}),
 
-               ('disable-msg',
+               ('disable',
                 {'type' : 'csv', 'metavar': '<msg ids>',
+                 'short': 'd',
                  'group': 'Messages control',
-                 'help' : 'Disable the message(s) with the given id(s).'}),
+                 'help' : 'Disable the message, report, category or checker with the '
+                 'given id(s). You can either give multiple identifier '
+                 'separated by comma (,) or put this option multiple time.'}),
                )
 
     option_groups = (
@@ -256,12 +236,10 @@ This is used by the global evaluation report (R0004).'}),
         self.options = options + PyLinter.options
         self.option_groups = option_groups + PyLinter.option_groups
         self._options_methods = {
-            'enable-report': self.enable_report,
-            'disable-report': self.disable_report,
-            'enable-msg': self.enable_message,
-            'disable-msg': self.disable_message,
-            'enable-msg-cat': self.enable_message_category,
-            'disable-msg-cat': self.disable_message_category}
+            'enable': self.enable,
+            'disable': self.disable}
+        self._bw_options_methods = {'disable-msg': self.disable,
+                                    'enable-msg': self.enable}
         full_version = '%%prog %s, \nastng %s, common %s\nPython %s' % (
             version, astng_version, common_version, sys.version)
         OptionsManagerMixIn.__init__(self, usage=__doc__,
@@ -271,13 +249,13 @@ This is used by the global evaluation report (R0004).'}),
         ReportsHandlerMixIn.__init__(self)
         BaseRawChecker.__init__(self)
         # provided reports
-        self.reports = (('R0001', 'Messages by category',
+        self.reports = (('RP0001', 'Messages by category',
                          report_total_messages_stats),
-                        ('R0002', '% errors / warnings by module',
+                        ('RP0002', '% errors / warnings by module',
                          report_messages_by_module_stats),
-                        ('R0003', 'Messages',
+                        ('RP0003', 'Messages',
                          report_messages_stats),
-                        ('R0004', 'Global evaluation',
+                        ('RP0004', 'Global evaluation',
                          self.report_evaluation),
                         )
         self.register_checker(self)
@@ -301,31 +279,31 @@ This is used by the global evaluation report (R0004).'}),
         self.reporter = reporter
         reporter.linter = self
 
-    def set_option(self, opt_name, value, action=None, opt_dict=None):
+    def set_option(self, optname, value, action=None, optdict=None):
         """overridden from configuration.OptionsProviderMixin to handle some
         special options
         """
-        if opt_name in self._options_methods:
+        if optname in self._options_methods or optname in self._bw_options_methods:
             if value:
-                meth = self._options_methods[opt_name]
-                value = check_csv(None, opt_name, value)
+                try:
+                    meth = self._options_methods[optname]
+                except KeyError:
+                    meth = self._bw_options_methods[optname]
+                    warn('%s is deprecated, replace it by %s' % (
+                        optname, optname.split('-')[0]), DeprecationWarning)
+                value = check_csv(None, optname, value)
                 if isinstance(value, (list, tuple)):
                     for _id in value :
                         meth(_id)
                 else :
                     meth(value)
-        elif opt_name == 'output-format':
+        elif optname == 'output-format':
             self.set_reporter(REPORTER_OPT_MAP[value.lower()]())
-        elif opt_name in ('enable-checker', 'disable-checker'):
-            if not value:
-                return
-            checkerids = [v.lower() for v in check_csv(None, opt_name, value)]
-            self.enable_checkers(checkerids, opt_name == 'enable-checker')
         try:
-            BaseRawChecker.set_option(self, opt_name, value, action, opt_dict)
+            BaseRawChecker.set_option(self, optname, value, action, optdict)
         except UnsupportedAction:
             print >> sys.stderr, 'option %s can\'t be read from config file' % \
-                  opt_name
+                  optname
 
     # checkers manipulation methods ############################################
 
@@ -335,7 +313,7 @@ This is used by the global evaluation report (R0004).'}),
         checker is an object implementing IRawChecker or / and IASTNGChecker
         """
         assert checker.priority <= 0, 'checker priority can\'t be >= 0'
-        self._checkers[checker.name] = checker
+        self._checkers.setdefault(checker.name, []).append(checker)
         if hasattr(checker, 'reports'):
             for r_id, r_title, r_cb in checker.reports:
                 self.register_report(r_id, r_title, r_cb, checker)
@@ -344,38 +322,14 @@ This is used by the global evaluation report (R0004).'}),
             self.register_messages(checker)
         checker.load_defaults()
 
-    def enable_checkers(self, listed, enabled):
-        """only enable/disable checkers from the given list"""
-        if enabled: # if we are activating a checker; deactivate them all first
-            for checker in self._checkers.values():
-                if not checker.may_be_disabled:
-                    continue
-                checker.enable(not enabled)
-        for checkerid in listed:
-            try:
-                checker = self._checkers[checkerid]
-            except KeyError:
-                raise Exception('no checker named %s' % checkerid)
-            checker.enable(enabled)
-
-    def disable_noerror_checkers(self):
-        """disable all checkers without error messages, and the
-        'miscellaneous' checker which can be safely deactivated in debug
-        mode
-        """
-        for checker in self._checkers.values():
-            if checker.name == 'miscellaneous':
-                checker.enable(False)
-                continue
-            # if checker is already explicitly disabled (e.g. rpython), don't
-            # enable it
-            if checker.enabled:
-                for msgid in getattr(checker, 'msgs', {}).keys():
-                    if msgid[0] == 'E':
-                        checker.enable(True)
-                        break
-                else:
-                    checker.enable(False)
+    def disable_noerror_messages(self):
+        for msgcat, msgids in self._msgs_by_category.iteritems():
+            if msgcat == 'E':
+                for msgid in msgids:
+                    self.enable(msgid)
+            else:
+                for msgid in msgids:
+                    self.disable(msgid)
 
     # block level option handling #############################################
     #
@@ -387,12 +341,9 @@ This is used by the global evaluation report (R0004).'}),
         """
         comment = tokenize.COMMENT
         newline = tokenize.NEWLINE
-        #line_num = 0
         for (tok_type, _, start, _, line) in tokens:
             if tok_type not in (comment, newline):
                 continue
-            #if start[0] == line_num:
-            #    continue
             match = OPTION_RGX.search(line)
             if match is None:
                 continue
@@ -407,9 +358,14 @@ This is used by the global evaluation report (R0004).'}),
                                  line=start[0])
                 continue
             opt = opt.strip()
-            #line_num = start[0]
-            if opt in self._options_methods and not opt.endswith('-report'):
-                meth = self._options_methods[opt]
+            if opt in self._options_methods or opt in self._bw_options_methods:
+                try:
+                    meth = self._options_methods[opt]
+                except KeyError:
+                    meth = self._bw_options_methods[opt]
+                    warn('%s is deprecated, replace it by %s (%s)' % (
+                        opt, opt.split('-')[0], self.current_file),
+                         DeprecationWarning)
                 for msgid in splitstrip(value):
                     try:
                         meth(msgid, 'module', start[0])
@@ -425,15 +381,15 @@ This is used by the global evaluation report (R0004).'}),
             self.collect_block_lines(child, msg_state)
         first = node.fromlineno
         last = node.tolineno
-        # first child line number used to distinguish between disable-msg
+        # first child line number used to distinguish between disable
         # which are the first child of scoped node with those defined later.
         # For instance in the code below:
         #
         # 1.   def meth8(self):
         # 2.        """test late disabling"""
-        # 3.        # pylint: disable-msg=E1102
+        # 3.        # pylint: disable=E1102
         # 4.        print self.blip
-        # 5.        # pylint: disable-msg=E1101
+        # 5.        # pylint: disable=E1101
         # 6.        print self.bla
         #
         # E1102 should be disabled from line 1 to 6 while E1101 from line 5 to 6
@@ -465,6 +421,36 @@ This is used by the global evaluation report (R0004).'}),
 
     # code checking methods ###################################################
 
+    def sort_checkers(self, checkers=None):
+        if checkers is None:
+            checkers = [checker for checkers in self._checkers.values()
+                        for checker in checkers]
+        graph = {}
+        cls_instance = {}
+        for checker in checkers:
+            graph[checker.__class__] = set(checker.needs_checkers)
+            cls_instance[checker.__class__] = checker
+        checkers = [cls_instance.get(cls) for cls in ordered_nodes(graph)]
+        checkers.remove(self)
+        checkers.insert(0, self)
+        return checkers
+
+    def _get_checkers(self):
+        # compute checkers needed according to activated messages and reports
+        neededcheckers = set()
+        for checkers in self._checkers.values():
+            for checker in checkers:
+                for msgid in checker.msgs:
+                    if self._msgs_state.get(msgid, True):
+                        neededcheckers.add(checker)
+                        break
+                else:
+                    for reportid, _, _ in checker.reports:
+                        if self.is_report_enabled(reportid):
+                            neededcheckers.add(checker)
+                            break
+        return self.sort_checkers(neededcheckers)
+
     def check(self, files_or_modules):
         """main checking entry: check a list of files or modules from their
         name.
@@ -472,10 +458,16 @@ This is used by the global evaluation report (R0004).'}),
         self.reporter.include_ids = self.config.include_ids
         if not isinstance(files_or_modules, (list, tuple)):
             files_or_modules = (files_or_modules,)
-        checkers = sort_checkers(self._checkers.values())
+        checkers = self._get_checkers()
+        rawcheckers = []
+        walker = PyLintASTWalker()
         # notify global begin
         for checker in checkers:
             checker.open()
+            if implements(checker, IASTNGChecker):
+                walker.add_checker(checker)
+            if implements(checker, IRawChecker) and checker is not self: #XXX
+                rawcheckers.append(checker)
         # build ast and check modules or packages
         for descr in self.expand_files(files_or_modules):
             modname, filepath = descr['name'], descr['path']
@@ -493,11 +485,12 @@ This is used by the global evaluation report (R0004).'}),
             # fix the current file (if the source file was not available or
             # if it's actually a c extension)
             self.current_file = astng.file
-            self.check_astng_module(astng, checkers)
+            self.check_astng_module(astng, walker, rawcheckers)
         # notify global end
         self.set_current_module('')
+        self.stats['statement'] = walker.nbstatements
         checkers.reverse()
-        for checker in  checkers:
+        for checker in checkers:
             checker.close()
 
     def expand_files(self, modules):
@@ -546,8 +539,7 @@ This is used by the global evaluation report (R0004).'}),
             #    traceback.print_exc()
             self.add_message('F0002', args=(ex.__class__, ex))
 
-
-    def check_astng_module(self, astng, checkers):
+    def check_astng_module(self, astng, walker, rawcheckers):
         """check a module from its astng representation, real work"""
         # call raw checkers if possible
         if not astng.pure_python:
@@ -564,41 +556,19 @@ This is used by the global evaluation report (R0004).'}),
             orig_state = self._module_msgs_state.copy()
             self._module_msgs_state = {}
             self.collect_block_lines(astng, orig_state)
-            for checker in checkers:
-                if implements(checker, IRawChecker) and checker is not self:
-                    stream.seek(0)
-                    checker.process_module(stream)
+            for checker in rawcheckers:
+                stream.seek(0)
+                checker.process_module(stream)
         # generate events to astng checkers
-        self.astng_events(astng, [checker for checker in checkers
-                                  if implements(checker, IASTNGChecker)])
+        walker.walk(astng)
         return True
 
-    def astng_events(self, astng, checkers, _reversed_checkers=None):
-        """generate event to astng checkers according to the current astng
-        node and recurse on its children
-        """
-        if _reversed_checkers is None:
-            _reversed_checkers = checkers[:]
-            _reversed_checkers.reverse()
-        if astng.is_statement:
-            self.stats['statement'] += 1
-        # generate events for this node on each checker
-        for checker in checkers:
-            checker.visit(astng)
-        # recurse on children
-        for child in astng.get_children():
-            self.astng_events(child, checkers, _reversed_checkers)
-        for checker in _reversed_checkers:
-            checker.leave(astng)
-
-
     # IASTNGChecker interface #################################################
 
     def open(self):
         """initialize counters"""
         self.stats = { 'by_module' : {},
                        'by_msg' : {},
-                       'statement' : 0
                        }
         for msg_cat in MSG_TYPES.values():
             self.stats[msg_cat] = 0
@@ -613,6 +583,8 @@ This is used by the global evaluation report (R0004).'}),
             old_stats = config.load_results(self.base_name)
             if self.config.reports:
                 self.make_reports(self.stats, old_stats)
+            elif self.config.output_format == 'html':
+                self.reporter.display_results(Section())
             # save results if persistent run
             if self.config.persistent:
                 config.save_results(self.stats, self.base_name)
@@ -701,7 +673,6 @@ def report_messages_by_module_stats(sect, stats, _):
     sect.append(Table(children=lines, cols=5, rheaders=1))
 
 
-
 # utilities ###################################################################
 
 # this may help to import modules using gettext
@@ -750,7 +721,7 @@ class Run:
 group are mutually exclusive.'),
         )
 
-    def __init__(self, args, reporter=None):
+    def __init__(self, args, reporter=None, exit=True):
         self._rcfile = None
         self._plugins = []
         preprocess_options(args, {
@@ -766,7 +737,7 @@ group are mutually exclusive.'),
 
             ('init-hook',
              {'action' : 'callback', 'type' : 'string', 'metavar': '<code>',
-              'callback' : cb_init_hook,
+              'callback' : cb_init_hook, 'level': 1,
               'help' : 'Python code to execute, usually for sys.path \
 manipulation such as pygtk.require().'}),
 
@@ -780,13 +751,13 @@ exit. The value may be a comma separated list of message ids.'''}),
             ('list-msgs',
              {'action' : 'callback', 'metavar': '<msg-id>',
               'callback' : self.cb_list_messages,
-              'group': 'Commands',
+              'group': 'Commands', 'level': 1,
               'help' : "Generate pylint's messages."}),
 
             ('full-documentation',
              {'action' : 'callback', 'metavar': '<msg-id>',
               'callback' : self.cb_full_documentation,
-              'group': 'Commands',
+              'group': 'Commands', 'level': 1,
               'help' : "Generate pylint's full documentation."}),
 
             ('generate-rcfile',
@@ -799,18 +770,18 @@ them in the generated configuration.'''}),
             ('generate-man',
              {'action' : 'callback', 'callback' : self.cb_generate_manpage,
               'group': 'Commands',
-              'help' : "Generate pylint's man page.",'hide': 'True'}),
+              'help' : "Generate pylint's man page.",'hide': True}),
 
             ('errors-only',
              {'action' : 'callback', 'callback' : self.cb_error_mode,
-              'short': 'e',
+              'short': 'E',
               'help' : '''In error mode, checkers without error messages are \
 disabled and for others, only the ERROR messages are displayed, and no reports \
 are done by default'''}),
 
             ('profile',
              {'type' : 'yn', 'metavar' : '<y_or_n>',
-              'default': False,
+              'default': False, 'hide': True,
               'help' : 'Profiled execution.'}),
 
             ), option_groups=self.option_groups,
@@ -821,7 +792,7 @@ are done by default'''}),
         # load command line plugins
         linter.load_plugin_modules(self._plugins)
         # add some help section
-        linter.add_help_section('Environment variables', config.ENV_HELP)
+        linter.add_help_section('Environment variables', config.ENV_HELP, level=1)
         linter.add_help_section('Output', '''
 Using the default text output, the message format is :                          
                                                                                 
@@ -834,7 +805,7 @@ There are 5 kind of message types :
     * (E) error, for probable bugs in the code                                  
     * (F) fatal, if an error occurred which prevented pylint from doing further
 processing.
-        ''')
+        ''', level=1)
         linter.add_help_section('Output status code', '''
 Pylint should leave with following status code:                                 
     * 0 if everything went fine                                                 
@@ -847,12 +818,12 @@ Pylint should leave with following status code:
                                                                                 
 status 1 to 16 will be bit-ORed so you can know which different categories has
 been issued by analysing pylint output status code
-        ''')
+        ''', level=1)
         # read configuration
-        linter.disable_message('W0704')
+        linter.disable('W0704')
         linter.read_config_file()
         # is there some additional plugins in the file configuration, in
-        config_parser = linter._config_parser
+        config_parser = linter.cfgfile_parser
         if config_parser.has_option('MASTER', 'load-plugins'):
             plugins = splitstrip(config_parser.get('MASTER', 'load-plugins'))
             linter.load_plugin_modules(plugins)
@@ -884,7 +855,8 @@ been issued by analysing pylint output status code
         else:
             linter.check(args)
         sys.path.pop(0)
-        sys.exit(self.linter.msg_status)
+        if exit:
+            sys.exit(self.linter.msg_status)
 
     def cb_set_rcfile(self, name, value):
         """callback for option preprocessing (i.e. before optik parsing)"""
@@ -896,13 +868,14 @@ been issued by analysing pylint output status code
 
     def cb_error_mode(self, *args, **kwargs):
         """error mode:
-        * checkers without error messages are disabled
-        * for others, only the ERROR messages are displayed
+        * disable all but error messages
+        * disable the 'miscellaneous' checker which can be safely deactivated in
+          debug
         * disable reports
         * do not save execution information
         """
-        self.linter.disable_noerror_checkers()
-        self.linter.set_option('disable-msg-cat', 'WCRI')
+        self.linter.disable_noerror_messages()
+        self.linter.disable('miscellaneous')
         self.linter.set_option('reports', False)
         self.linter.set_option('persistent', False)
 
@@ -917,22 +890,22 @@ been issued by analysing pylint output status code
         self.linter.generate_manpage(__pkginfo__)
         sys.exit(0)
 
-    def cb_help_message(self, option, opt_name, value, parser):
+    def cb_help_message(self, option, optname, value, parser):
         """optik callback for printing some help about a particular message"""
         self.linter.help_message(splitstrip(value))
         sys.exit(0)
 
-    def cb_full_documentation(self, option, opt_name, value, parser):
+    def cb_full_documentation(self, option, optname, value, parser):
         """optik callback for printing full documentation"""
         self.linter.print_full_documentation()
         sys.exit(0)
 
-    def cb_list_messages(self, option, opt_name, value, parser): # FIXME
+    def cb_list_messages(self, option, optname, value, parser): # FIXME
         """optik callback for printing available messages"""
-        self.linter.list_sorted_messages()
+        self.linter.list_messages()
         sys.exit(0)
 
-def cb_init_hook(option, opt_name, value, parser):
+def cb_init_hook(option, optname, value, parser):
     """exec arbitrary code to set sys.path for instance"""
     exec value
 
