@@ -1,4 +1,4 @@
-# Copyright (c) 2000-2008 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2000-2010 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -95,7 +95,8 @@ class DiaDefGenerator:
         """return associated nodes of a class node"""
         if level == 0:
             return
-        for ass_nodes in klass_node.instance_attrs_type.values():
+        for ass_nodes in klass_node.instance_attrs_type.values() + \
+                         klass_node.locals_type.values():
             for ass_node in ass_nodes:
                 if isinstance(ass_node, astng.Instance):
                     ass_node = ass_node._proxied
