@@ -1,4 +1,4 @@
-# Copyright (c) 2004-2008 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2004-2010 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -80,7 +80,8 @@ class ClassDiagram(Figure, FilterMixIn):
     def get_attrs(self, node):
         """return visible attributes, possibly with class name"""
         attrs = []
-        for node_name, ass_nodes in node.instance_attrs_type.items():
+        for node_name, ass_nodes in node.instance_attrs_type.items() + \
+                                node.locals_type.items():
             if not self.show_attr(node_name):
                 continue
             names = self.class_names(ass_nodes)
@@ -169,7 +170,8 @@ class ClassDiagram(Figure, FilterMixIn):
                 except KeyError:
                     continue
             # associations link
-            for name, values in node.instance_attrs_type.items():
+            for name, values in node.instance_attrs_type.items() + \
+                                node.locals_type.items():
                 for value in values:
                     if value is astng.YES:
                         continue
@@ -180,7 +182,8 @@ class ClassDiagram(Figure, FilterMixIn):
                         self.add_relationship(ass_obj, obj, 'association', name)
                     except KeyError:
                         continue
-        
+
+
 class PackageDiagram(ClassDiagram):
     """package diagram handling
     """
