@@ -14,7 +14,7 @@ def titii():
 
 def tataa(kdict):
     """blank line unindented"""
-    for key in [1, 2, 3]:
+    for key in ['1', '2', '3']:
         key = key.lower()
     
         if kdict.has_key(key):
