@@ -1,4 +1,4 @@
-#pylint: disable-msg=C0111,C0321
+#pylint: disable=C0111,C0321
 """pylint complains about 'index' being used before definition"""
 
 __revision__ = None
@@ -18,3 +18,6 @@ import sys; print sys.exc_info( )
 for i in range(10): print i
 
 j = 4; LAMB = lambda x: x+j
+
+FUNC4 = lambda a, b : a != b
+FUNC3 = lambda (a, b) : a != b
