@@ -1,4 +1,4 @@
-# pylint: disable-msg=R0903,W0212
+# pylint: disable=R0903,R0201
 """test Invalid name"""
 
 __revision__ = 1
@@ -10,7 +10,7 @@ def Run():
         def __init__(self):
             pass
     bBb = 1
-    return A(bBb)
+    return A, bBb
 
 def run():
     """anothrer method without only good name"""
@@ -40,3 +40,15 @@ class xyz:
 def nested_args(arg1, (arg21, arg22)):
     """function with nested arguments"""
     print arg1, arg21, arg22
+
+
+GOOD_CONST_NAME = ''
+benpasceluila = 0
+
+class Correct: 
+    """yo"""
+    def __init__(self):
+        self.cava = 12
+        self._Ca_va_Pas = None
+
+V = [WHAT_Ever_inListComp for WHAT_Ever_inListComp in GOOD_CONST_NAME]
