@@ -3,5 +3,5 @@
 
 __revision__ = apply(map, (str, (1, 2, 3)))
 
-xxxx = map(str, (1, 2, 3))
+YYYYY = map(str, (1, 2, 3))
 
