@@ -38,8 +38,7 @@ checkers.initialize(linter)
 linter.global_set_option('required-attributes', ('__revision__',))
 
 PY23 = sys.version_info >= (2, 3)
-PY24 = sys.version_info >= (2, 4)
-PY25 = sys.version_info >= (2, 5)
+PY26 = sys.version_info >= (2, 6)
 
 
 if linesep != '\n':
@@ -56,7 +55,7 @@ def exception_str(ex):
     """function used to replace default __str__ method of exception instances"""
     return 'in %s\n:: %s' % (ex.file, ', '.join(ex.args))
 
-class LintTestUsingModule(testlib.TestCase):            
+class LintTestUsingModule(testlib.TestCase):
     DEFAULT_PACKAGE = 'input'
     package = DEFAULT_PACKAGE
     linter = linter
@@ -83,12 +82,12 @@ class LintTestUsingModule(testlib.TestCase):
             tocheck += [self.package+'.%s' % name.replace('.py', '')
                         for name, file in self.depends]
         self._test(tocheck)
-        
+
     def _test(self, tocheck):
         if INFO_TEST_RGX.match(self.module):
-            self.linter.enable_message_category('I')
+            self.linter.enable('I')
         else:
-            self.linter.disable_message_category('I')
+            self.linter.disable('I')
         try:
             self.linter.check(tocheck)
         except Exception, ex:
@@ -112,8 +111,8 @@ class LintTestUsingModule(testlib.TestCase):
             #ex.__str__ = new.instancemethod(exception_str, ex, None)
             raise AssertionError('%s: %s' % (self.module, ex)), None, sys.exc_info()[-1]
 
-class LintTestUsingFile(LintTestUsingModule):            
-                
+class LintTestUsingFile(LintTestUsingModule):
+
     _TEST_TYPE = 'file'
 
     def test_functionality(self):
@@ -135,8 +134,8 @@ class TestTests(testlib.TestCase):
             except ValueError:
                 continue
         todo.sort()
-        if PY25:
-            self.assertEqual(todo, ['E0503', 'I0001'])
+        if PY26:
+            self.assertEqual(todo, ['E0503', 'E1122', 'I0001'])
         elif PY23:
             self.assertEqual(todo, ['E0503', 'I0001'])
         else: # python < 2.3
@@ -161,7 +160,7 @@ class TestTests(testlib.TestCase):
 
 def make_tests(filter_rgx):
     """generate tests classes from test info
-    
+
     return the list of generated test classes
     """
     if filter_rgx:
@@ -177,11 +176,15 @@ def make_tests(filter_rgx):
         if pyrestr.isdigit(): # '24', '25'...
             if sys.version_info < tuple([int(i) for i in pyrestr]):
                 continue
+        if pyrestr.startswith('_') and  pyrestr[1:].isdigit():
+            # skip test for higher python versions
+            if sys.version_info >= ( int(pyrestr[1]), int(pyrestr[2]) ):
+                continue
         if not is_to_run(module_file):
             continue
         base = module_file.replace('func_', '').replace('.py', '')
         dependencies = get_tests_info(base, '.py')
-        
+
         class LintTestUsingModuleTC(LintTestUsingModule):
             module = module_file.replace('.py', '')
             output = messages_file
@@ -191,14 +194,14 @@ def make_tests(filter_rgx):
 
         if MODULES_ONLY:
             continue
-        
+
         class LintTestUsingFileTC(LintTestUsingFile):
             module = module_file.replace('.py', '')
             output = exists(messages_file + '2') and (messages_file + '2') or messages_file
             depends = dependencies or None
             tags = testlib.Tags(('generated', 'pylint_input_%s' % module))
         tests.append(LintTestUsingFileTC)
-        
+
 ##     # special test for f0003
 ##     module_file, messages_file in get_tests_info('func_f0003', '.pyc')
 ##     class LintTestSubclass(LintTest):
@@ -206,16 +209,16 @@ def make_tests(filter_rgx):
 ##         output = messages_file
 ##         depends = dependencies or None
 ##     tests.append(LintTestSubclass)
-            
+
     class LintBuiltinModuleTest(LintTestUsingModule):
         output = 'messages/builtin_module.txt'
         module = 'sys'
         def test_functionality(self):
             self._test(['sys'])
     tests.append(LintBuiltinModuleTest)
-    
+
     if not filter_rgx:
-        # test all features are tested :)    
+        # test all features are tested :)
         tests.append(TestTests)
 
     return tests
@@ -231,8 +234,8 @@ if __name__=='__main__':
     if '-m' in sys.argv:
         MODULES_ONLY = True
         sys.argv.remove('-m')
-    
-    if len(sys.argv) > 1:            
+
+    if len(sys.argv) > 1:
         FILTER_RGX = sys.argv[1]
         del sys.argv[1]
     testlib.unittest_main(defaultTest='suite')
