@@ -15,7 +15,7 @@ class DependenciesGraphTC(unittest.TestCase):
     dest = 'dependencies_graph.dot'
     def tearDown(self):
         os.remove(self.dest)
-        
+
     def test_dependencies_graph(self):
         imports.dependencies_graph(self.dest, {'labas': ['hoho', 'yep'],
                                                'hoho': ['yep']})
@@ -33,16 +33,16 @@ URL="." node[shape="box"]
 "yep" -> "labas" [];
 }
 '''.strip())
-                          
+
 class ImportCheckerTC(unittest.TestCase):
     def setUp(self):
         self.linter = l = PyLinter(reporter=TestReporter())
         initialize(l)
-        
+
     def test_checker_dep_graphs(self):
         l = self.linter
         l.global_set_option('persistent', False)
-        l.global_set_option('enable-checker', 'imports')
+        l.global_set_option('enable', 'imports')
         l.global_set_option('import-graph', 'import.dot')
         l.global_set_option('ext-import-graph', 'ext_import.dot')
         l.global_set_option('int-import-graph', 'int_import.dot')
@@ -60,6 +60,6 @@ class ImportCheckerTC(unittest.TestCase):
                     os.remove(fname)
                 except:
                     pass
-                    
+
 if __name__ == '__main__':
     unittest.main()
