@@ -1,5 +1,5 @@
-# Copyright (c) 2003-2009 Sylvain Thenault (thenault@gmail.com).
-# Copyright (c) 2003-2009 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2010 Sylvain Thenault (thenault@gmail.com).
+# Copyright (c) 2003-2010 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -20,12 +20,15 @@ main pylint class
 
 import sys
 from os import linesep
+from os.path import dirname, basename, splitext, exists, isdir, join, normpath
 
+from logilab.common.modutils import modpath_from_file, get_module_files, \
+                                    file_from_modpath
 from logilab.common.textutils import normalize_text
 from logilab.common.configuration import rest_format_section
 from logilab.common.ureports import Section
 
-from logilab.astng import Module
+from logilab.astng import nodes, Module
 
 from pylint.checkers import EmptyReport
 
@@ -41,6 +44,8 @@ MSG_TYPES = {
     'E' : 'error',
     'F' : 'fatal'
     }
+MSG_TYPES_LONG = dict([(v, k) for k, v in MSG_TYPES.iteritems()])
+
 MSG_TYPES_STATUS = {
     'I' : 0,
     'C' : 16,
@@ -50,13 +55,7 @@ MSG_TYPES_STATUS = {
     'F' : 1
     }
 
-def sort_checkers(checkers):
-    """return a list of enabled checker sorted by priority"""
-    checkers = [checker for checker in checkers if checker.is_enabled()]
-    checkers.sort(lambda x, y: cmp(-x.priority, -y.priority) )
-    return checkers
-
-def sort_msgs(msg_ids):
+def sort_msgs(msgids):
     """sort message identifiers according to their category first"""
     msg_order = 'EWRCIF'
     def cmp_func(msgid1, msgid2):
@@ -65,8 +64,8 @@ def sort_msgs(msg_ids):
             return cmp(msg_order.index(msgid1[0]), msg_order.index(msgid2[0]))
         else:
             return cmp(msgid1, msgid2)
-    msg_ids.sort(cmp_func)
-    return msg_ids
+    msgids.sort(cmp_func)
+    return msgids
 
 def get_module_and_frameid(node):
     """return the module name and the frame id in the module"""
@@ -84,6 +83,12 @@ def get_module_and_frameid(node):
     obj.reverse()
     return module, '.'.join(obj)
 
+def category_id(id):
+    id = id.upper()
+    if id in MSG_TYPES:
+        return id
+    return MSG_TYPES_LONG.get(id)
+
 
 class Message:
     def __init__(self, checker, msgid, msg, descr):
@@ -105,8 +110,7 @@ class MessagesHandlerMixIn:
         self._messages = {}
         self._msgs_state = {}
         self._module_msgs_state = {} # None
-        self._msg_cats_state = {}
-        self._module_msg_cats_state = None
+        self._msgs_by_category = {}
         self.msg_status = 0
 
     def register_messages(self, checker):
@@ -128,10 +132,11 @@ class MessagesHandlerMixIn:
                    'Inconsistent checker part in message id %r' % msgid
             chkid = msgid[1:3]
             self._messages[msgid] = Message(checker, msgid, msg, msgdescr)
+            self._msgs_by_category.setdefault(msgid[0], []).append(msgid)
 
-    def get_message_help(self, msg_id, checkerref=False):
+    def get_message_help(self, msgid, checkerref=False):
         """return the help string for the given message id"""
-        msg = self.check_message_id(msg_id)
+        msg = self.check_message_id(msgid)
         desc = normalize_text(' '.join(msg.descr.split()), indent='  ')
         if checkerref:
             desc += ' This message belongs to the %s checker.' % \
@@ -142,17 +147,34 @@ class MessagesHandlerMixIn:
             return ':%s: *%s*\n%s' % (msg.msgid, title, desc)
         return ':%s:\n%s' % (msg.msgid, desc)
 
-    def disable_message(self, msg_id, scope='package', line=None):
+    def disable(self, msgid, scope='package', line=None):
         """don't output message of the given id"""
         assert scope in ('package', 'module')
-        msg = self.check_message_id(msg_id)
+        # msgid is a category?
+        catid = category_id(msgid)
+        if catid is not None:
+            for msgid in self._msgs_by_category.get(catid):
+                self.disable(msgid, scope, line)
+            return
+        # msgid is a checker name?
+        if msgid.lower() in self._checkers:
+            for checker in self._checkers[msgid.lower()]:
+                for msgid in checker.msgs:
+                    self.disable(msgid, scope, line)
+            return
+        # msgid is report id?
+        if msgid.lower().startswith('rp'):
+            self.disable_report(msgid)
+            return
+        # msgid is a msgid.
+        msg = self.check_message_id(msgid)
         if scope == 'module':
             assert line > 0
             try:
                 self._module_msgs_state[msg.msgid][line] = False
             except KeyError:
                 self._module_msgs_state[msg.msgid] = {line: False}
-                if msg_id != 'I0011':
+                if msgid != 'I0011':
                     self.add_message('I0011', line=line, args=msg.msgid)
 
         else:
@@ -162,11 +184,27 @@ class MessagesHandlerMixIn:
             self.config.disable_msg = [mid for mid, val in msgs.items()
                                        if not val]
 
-    def enable_message(self, msg_id, scope='package', line=None):
+    def enable(self, msgid, scope='package', line=None):
         """reenable message of the given id"""
         assert scope in ('package', 'module')
-        msg = self.check_message_id(msg_id)
-        msg.checker.enabled = True # ensure the related checker is enabled
+        catid = category_id(msgid)
+        # msgid is a category?
+        if catid is not None:
+            for msgid in self._msgs_by_category.get(catid):
+                self.enable(msgid, scope, line)
+            return
+        # msgid is a checker name?
+        if msgid.lower() in self._checkers:
+            for checker in self._checkers[msgid.lower()]:
+                for msgid in checker.msgs:
+                    self.enable(msgid, scope, line)
+            return
+        # msgid is report id?
+        if msgid.lower().startswith('rp'):
+            self.enable_report(msgid)
+            return
+        # msgid is a msgid.
+        msg = self.check_message_id(msgid)
         if scope == 'module':
             assert line > 0
             try:
@@ -178,61 +216,28 @@ class MessagesHandlerMixIn:
             msgs = self._msgs_state
             msgs[msg.msgid] = True
             # sync configuration object
-            self.config.enable_msg = [mid for mid, val in msgs.items() if val]
+            self.config.enable = [mid for mid, val in msgs.items() if val]
 
-    def _cat_ids(self, categories):
-        for catid in categories:
-            catid = catid.upper()
-            if not catid in MSG_TYPES:
-                raise Exception('Unknown category identifier %s' % catid)
-            yield catid
-
-    def disable_message_category(self, categories, scope='package', line=None):
-        """don't output message in the given category"""
-        assert scope in ('package', 'module')
-        for catid in self._cat_ids(categories):
-            if scope == 'module':
-                self.add_message('I0011', line=line, args=catid)
-                self._module_msg_cats_state[catid] = False
-            else:
-                self._msg_cats_state[catid] = False
-
-    def enable_message_category(self, categories, scope='package', line=None):
-        """reenable message of the given category"""
-        assert scope in ('package', 'module')
-        for catid in self._cat_ids(categories):
-            if scope == 'module':
-                self.add_message('I0012', line=line, args=catid)
-                self._module_msg_cats_state[catid] = True
-            else:
-                self._msg_cats_state[catid] = True
-
-    def check_message_id(self, msg_id):
+    def check_message_id(self, msgid):
         """raise UnknownMessage if the message id is not defined"""
-        msg_id = msg_id.upper()
+        msgid = msgid.upper()
         try:
-            return self._messages[msg_id]
+            return self._messages[msgid]
         except KeyError:
-            raise UnknownMessage('No such message id %s' % msg_id)
+            raise UnknownMessage('No such message id %s' % msgid)
 
-    def is_message_enabled(self, msg_id, line=None):
+    def is_message_enabled(self, msgid, line=None):
         """return true if the message associated to the given message id is
         enabled
         """
-        try:
-            if not self._module_msg_cats_state[msg_id[0]]:
-                return False
-        except (KeyError, TypeError):
-            if not self._msg_cats_state.get(msg_id[0], True):
-                return False
         if line is None:
-            return self._msgs_state.get(msg_id, True)
+            return self._msgs_state.get(msgid, True)
         try:
-            return self._module_msgs_state[msg_id][line]
+            return self._module_msgs_state[msgid][line]
         except (KeyError, TypeError):
-            return self._msgs_state.get(msg_id, True)
+            return self._msgs_state.get(msgid, True)
 
-    def add_message(self, msg_id, line=None, node=None, args=None):
+    def add_message(self, msgid, line=None, node=None, args=None):
         """add the message corresponding to the given id.
 
         If provided, msg is expanded using args
@@ -243,18 +248,18 @@ class MessagesHandlerMixIn:
         if line is None and node is not None:
             line = node.fromlineno
         # should this message be displayed
-        if not self.is_message_enabled(msg_id, line):
+        if not self.is_message_enabled(msgid, line):
             return
         # update stats
-        msg_cat = MSG_TYPES[msg_id[0]]
-        self.msg_status |= MSG_TYPES_STATUS[msg_id[0]]
+        msg_cat = MSG_TYPES[msgid[0]]
+        self.msg_status |= MSG_TYPES_STATUS[msgid[0]]
         self.stats[msg_cat] += 1
         self.stats['by_module'][self.current_name][msg_cat] += 1
         try:
-            self.stats['by_msg'][msg_id] += 1
+            self.stats['by_msg'][msgid] += 1
         except KeyError:
-            self.stats['by_msg'][msg_id] = 1
-        msg = self._messages[msg_id].msg
+            self.stats['by_msg'][msgid] = 1
+        msg = self._messages[msgid].msg
         # expand message ?
         if args:
             msg %= args
@@ -266,22 +271,23 @@ class MessagesHandlerMixIn:
             module, obj = get_module_and_frameid(node)
             path = node.root().file
         # add the message
-        self.reporter.add_message(msg_id, (path, module, obj, line or 1), msg)
+        self.reporter.add_message(msgid, (path, module, obj, line or 1), msg)
 
     def help_message(self, msgids):
         """display help messages for the given message identifiers"""
-        for msg_id in msgids:
+        for msgid in msgids:
             try:
-                print self.get_message_help(msg_id, True)
+                print self.get_message_help(msgid, True)
                 print
             except UnknownMessage, ex:
                 print ex
                 print
                 continue
 
-    def list_messages(self):
+    def print_full_documentation(self):
         """output a full documentation in ReST format"""
-        for checker in sort_checkers(self._checkers.values()):
+        by_checker = {}
+        for checker in self.sort_checkers():
             if checker.name == 'master':
                 prefix = 'Main '
                 if checker.options:
@@ -295,36 +301,52 @@ class MessagesHandlerMixIn:
                         rest_format_section(sys.stdout, None, options)
                         print
             else:
-                prefix = ''
-                title = '%s checker' % checker.name.capitalize()
+                try:
+                    by_checker[checker.name][0] += checker.options_and_values()
+                    by_checker[checker.name][1].update(checker.msgs)
+                    by_checker[checker.name][2] += checker.reports
+                except KeyError:
+                    by_checker[checker.name] = [list(checker.options_and_values()),
+                                                dict(checker.msgs),
+                                                list(checker.reports)]
+        for checker, (options, msgs, reports) in by_checker.items():
+            prefix = ''
+            title = '%s checker' % checker
+            print title
+            print '-' * len(title)
+            if options:
+                title = 'Options'
                 print title
-                print '-' * len(title)
-                if checker.__doc__: # __doc__ is None with -OO
-                    print linesep.join([l.strip()
-                                        for l in checker.__doc__.splitlines()])
-                if checker.options:
-                    title = 'Options'
-                    print title
-                    print '~' * len(title)
-                    for section, options in checker.options_by_section():
-                        rest_format_section(sys.stdout, section, options)
-                        print
-            if checker.msgs:
+                print '~' * len(title)
+                rest_format_section(sys.stdout, None, options)
+                print
+            if msgs:
                 title = ('%smessages' % prefix).capitalize()
                 print title
                 print '~' * len(title)
-                for msg_id in sort_msgs(checker.msgs.keys()):
-                    print self.get_message_help(msg_id, False)
+                for msgid in sort_msgs(msgs.keys()):
+                    print self.get_message_help(msgid, False)
                 print
-            if getattr(checker, 'reports', None):
+            if reports:
                 title = ('%sreports' % prefix).capitalize()
                 print title
                 print '~' * len(title)
-                for report in checker.reports:
+                for report in reports:
                     print ':%s: %s' % report[:2]
                 print
             print
 
+    def list_messages(self):
+        """output full messages list documentation in ReST format"""
+        msgids = []
+        for checker in self.sort_checkers():
+            for msgid in checker.msgs.keys():
+                msgids.append(msgid)
+        msgids.sort()
+        for msgid in msgids:
+            print self.get_message_help(msgid, False)
+        print
+
 
 class ReportsHandlerMixIn:
     """a mix-in class containing all the reports and stats manipulation
@@ -334,32 +356,32 @@ class ReportsHandlerMixIn:
         self._reports = {}
         self._reports_state = {}
 
-    def register_report(self, r_id, r_title, r_cb, checker):
+    def register_report(self, reportid, r_title, r_cb, checker):
         """register a report
 
-        r_id is the unique identifier for the report
+        reportid is the unique identifier for the report
         r_title the report's title
         r_cb the method to call to make the report
         checker is the checker defining the report
         """
-        r_id = r_id.upper()
-        self._reports.setdefault(checker, []).append( (r_id, r_title, r_cb) )
+        reportid = reportid.upper()
+        self._reports.setdefault(checker, []).append( (reportid, r_title, r_cb) )
 
-    def enable_report(self, r_id):
+    def enable_report(self, reportid):
         """disable the report of the given id"""
-        r_id = r_id.upper()
-        self._reports_state[r_id] = True
+        reportid = reportid.upper()
+        self._reports_state[reportid] = True
 
-    def disable_report(self, r_id):
+    def disable_report(self, reportid):
         """disable the report of the given id"""
-        r_id = r_id.upper()
-        self._reports_state[r_id] = False
+        reportid = reportid.upper()
+        self._reports_state[reportid] = False
 
-    def is_report_enabled(self, r_id):
+    def is_report_enabled(self, reportid):
         """return true if the report associated to the given identifier is
         enabled
         """
-        return self._reports_state.get(r_id, True)
+        return self._reports_state.get(reportid, True)
 
     def make_reports(self, stats, old_stats):
         """render registered reports"""
@@ -368,18 +390,18 @@ class ReportsHandlerMixIn:
             self.reporter.set_output(open(filename, 'w'))
         sect = Section('Report',
                        '%s statements analysed.'% (self.stats['statement']))
-        checkers = sort_checkers(self._reports.keys())
+        checkers = self.sort_checkers(self._reports)
         checkers.reverse()
         for checker in checkers:
-            for r_id, r_title, r_cb in self._reports[checker]:
-                if not self.is_report_enabled(r_id):
+            for reportid, r_title, r_cb in self._reports[checker]:
+                if not self.is_report_enabled(reportid):
                     continue
                 report_sect = Section(r_title)
                 try:
                     r_cb(report_sect, stats, old_stats)
                 except EmptyReport:
                     continue
-                report_sect.report_id = r_id
+                report_sect.report_id = reportid
                 sect.append(report_sect)
         self.reporter.display_results(sect)
 
@@ -394,11 +416,6 @@ class ReportsHandlerMixIn:
             self.stats[key] = value
         return self.stats
 
-###     ###     - - - -   import utils    - - - -     ###     ###
-
-from os.path import dirname, basename, splitext, exists, isdir, join, normpath
-from logilab.common.modutils import modpath_from_file, get_module_files, \
-                                    file_from_modpath
 
 def expand_modules(files_or_modules, black_list):
     """take a list of files/modules/packages and return the list of tuple
@@ -440,3 +457,62 @@ def expand_modules(files_or_modules, black_list):
                 result.append( {'path': subfilepath, 'name': submodname,
                                 'basepath': filepath, 'basename': modname} )
     return result, errors
+
+
+class PyLintASTWalker(object):
+    def __init__(self):
+        # callbacks per node types
+        self.nbstatements = 1
+        self.visit_events = {}
+        self.leave_events = {}
+
+    def add_checker(self, checker):
+        hasvdefault = False
+        vcids = set()
+        hasldefault = False
+        lcids = set()
+        for member in dir(checker):
+            if member.startswith('visit_'):
+                cid = member[6:]
+                if cid != 'default':
+                    cbs = self.visit_events.setdefault(cid, [])
+                    cbs.append(getattr(checker, member))
+                    vcids.add(cid)
+                else:
+                    hasvdefault = getattr(checker, member)
+            elif member.startswith('leave_'):
+                cid = member[6:]
+                if cid != 'default':
+                    cbs = self.leave_events.setdefault(cid, [])
+                    cbs.append(getattr(checker, member))
+                    lcids.add(cid)
+                else:
+                    hasldefault = getattr(checker, member)
+        if hasvdefault:
+            for cls in nodes.ALL_NODE_CLASSES:
+                cid = cls.__name__.lower()
+                if cid not in vcids:
+                    cbs = self.visit_events.setdefault(cid, [])
+                    cbs.append(hasvdefault)
+        if hasldefault:
+            for cls in nodes.ALL_NODE_CLASSES:
+                cid = cls.__name__.lower()
+                if cid not in lcids:
+                    cbs = self.leave_events.setdefault(cid, [])
+                    cbs.append(hasldefault)
+
+    def walk(self, astng):
+        """call visit events of astng checkers for the given node, recurse on
+        its children, then leave events.
+        """
+        cid = astng.__class__.__name__.lower()
+        if astng.is_statement:
+            self.nbstatements += 1
+        # generate events for this node on each checker
+        for cb in self.visit_events.get(cid, ()):
+            cb(astng)
+        # recurse on children
+        for child in astng.get_children():
+            self.walk(child)
+        for cb in self.leave_events.get(cid, ()):
+            cb(astng)
